package org.greenstone.gsdl3.util;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.w3c.dom.Text;

import java.util.HashMap;
//import java.util.Locale;

/** various functions for extracting info out of GS XML */
public class GSXML {

    // greenstone xml elements
    public static final String MESSAGE_ELEM = "message";
    public static final String REQUEST_ELEM = "request";
    public static final String RESPONSE_ELEM = "response";
    public static final String COLLECTION_ELEM = "collection";
    public static final String SERVICE_ELEM = "service";
    public static final String CLUSTER_ELEM = "serviceCluster";
    public static final String SITE_ELEM = "site";
    public static final String PARAM_ELEM = "param";
    public static final String PARAM_OPTION_ELEM = "option";
    public static final String CONTENT_ELEM = "content";
    public static final String RESOURCE_ELEM = "resource";
    public static final String DOCUMENT_ELEM = "document";
    public static final String METADATA_ELEM = "metadata";
    public static final String SERVICE_CLASS_ELEM = "serviceRack";
    public static final String CLASSIFIER_ELEM = "classifier";
    public static final String APPLET_ELEM = "applet";
    public static final String APPLET_DATA_ELEM = "appletData";
    public static final String CONFIGURE_ELEM = "configure";
    public static final String STATUS_ELEM = "status";
    public static final String ERROR_ELEM = "error";
    public static final String DEFAULT_ELEM = "default";
    public static final String STYLESHEET_ELEM = "format";//"stylesheet"; // any additional stylesheet stuff is carried in the message inside this elem
    public static final String FORMAT_ELEM = "format"; // config files use format - shoudl we use this instead of stylesheet??
    public static final String TERM_ELEM = "term";

    //config file elems
    public static final String INDEX_ELEM = "index";
    public static final String SEARCH_ELEM = "search";
    public static final String BROWSE_ELEM = "browse";
    public static final String FIELD_ELEM = "field";
    public static final String DISPLAYNAME_ELEM = "displayName";
    public static final String SHORTNAME_ATT = "shortname";

    // elems for the pages to be processed by xslt
    public final static String PAGE_ELEM = "page";
    public final static String TRANSLATION_ELEM = "translate";
    public final static String CONFIGURATION_ELEM = "config";
    public final static String PAGE_REQUEST_ELEM = "pageRequest";
    public final static String PAGE_RESPONSE_ELEM = "pageResponse";
    
    //public final static String DESCRIPTION_ELEM = "description";

    public static final String SITE_NAME_ELEM = "localSiteName";
    public static final String SITE_HTTP_ADDRESS_ELEM = "httpAddress";

    // add on to another elem type to get a list of that type
    public static final String LIST_MODIFIER = "List";

    // greenstone xml attributes
    public static final String NAME_ATT = "name";
    public static final String TO_ATT = "to";
    public static final String FROM_ATT = "from";
    public static final String LANG_ATT = "lang";
    public static final String TYPE_ATT = "type";
    public static final String VALUE_ATT = "value";
    public static final String DEFAULT_ATT = "default";
    public static final String INFO_ATT = "info";
    public static final String ACTION_ATT = "action";
    public static final String SUBACTION_ATT = "subaction";
    public static final String OUTPUT_ATT = "output";
    public static final String ADDRESS_ATT = "address";
    public static final String STATUS_ERROR_CODE_ATT = "code";
    public static final String STATUS_PROCESS_ID_ATT = "pid";
    public static final String PARAM_SHORTNAME_ATT = "shortname";
    public static final String PARAM_IGNORE_POS_ATT = "ignore";
    public static final String CLASSIFIER_CONTENT_ATT = "content";

    // document stuff
    public static final String DOC_NODE_ELEM = "documentNode";
    public static final String NODE_CONTENT_ELEM = "nodeContent";
    public static final String NODE_STRUCTURE_ELEM = "nodeStructure";
    public static final String NODE_ID_ATT = "nodeID";
    public static final String NODE_NAME_ATT = "nodeName";
    public static final String NODE_TYPE_ATT = "nodeType";

    public static final String NODE_TYPE_ROOT = "root";
    public static final String NODE_TYPE_INTERIOR = "interior";
    public static final String NODE_TYPE_LEAF = "leaf";

    // classifier stuff
    public static final String CLASS_NODE_ELEM = "classifierNode";
    public static final String CLASS_NODE_ORIENTATION_ATT = "orientation";
    
    // parameter types
    public static final String PARAM_TYPE_INTEGER = "integer";
    public static final String PARAM_TYPE_BOOLEAN = "boolean";
    public static final String PARAM_TYPE_ENUM_START = "enum";
    public static final String PARAM_TYPE_ENUM_SINGLE = "enum_single";
    public static final String PARAM_TYPE_ENUM_MULTI = "enum_multi";
    public static final String PARAM_TYPE_STRING = "string";
    public static final String PARAM_TYPE_TEXT = "text";
    public static final String PARAM_TYPE_MULTI = "multi";
    public static final String PARAM_TYPE_FILE = "file";
    // stuff for text strings 
    public static final String DISPLAY_ELEM = "display";
    public static final String DISPLAY_NAME_ELEM = "name";
    public static final String DISPLAY_SUBMIT_ELEM = "submit";

    // request types - do we need all these? or just
    // describe, status, and one to say do teh request
    public static final String REQUEST_TYPE_DESCRIBE = "describe";
    public static final String REQUEST_TYPE_STATUS = "status";
    public static final String REQUEST_TYPE_PROCESS = "process";
    public static final String REQUEST_TYPE_CONFIGURE = "configure"; 
    public static final String REQUEST_TYPE_CGI = "cgi"; 
    public static final String REQUEST_TYPE_FORMAT = "format"; 

    public static final String REQUEST_TYPE_ACTION = "action"; // ??
    public static final String REQUEST_TYPE_QUERY = "query"; // ??
    public static final String REQUEST_TYPE_BUILD = "build"; // ??
   
    // service types
    public static final String SERVICE_TYPE_QUERY = "query";
    public static final String SERVICE_TYPE_RETRIEVE = "retrieve";
    public static final String SERVICE_TYPE_BROWSE = "browse";
    public static final String SERVICE_TYPE_APPLET = "applet";
    public static final String SERVICE_TYPE_PROCESS = "process";

    // configure types
    public static final String CONFIG_ACTION_ACTIVATE = "activate";
    public static final String CONFIG_ACTION_DEACTIVATE = "deactivate";
   
    // communicator types
    public static final String COMM_TYPE_SOAP_JAVA = "soap";
    
    // a param - shoudl it go here?? 
    public static final String INFO_PARAM = "info";

    /** takes a list of elements, and returns an array of strings 
     * of the values of attribute att_name */
    public static String [] getAttributeValuesFromList(Element list,
						       String att_name) {
	
	NodeList children = list.getChildNodes();
	
	int num_nodes = children.getLength();
	String []ids = new String[num_nodes];
	for (int i=0; i<num_nodes; i++) {
	    Element e = (Element)children.item(i);
	    String id = e.getAttribute(att_name);	    
	    ids[i] = id;	    
	}
	
	return ids;
    }

    /** takes a paramList element, and gets a HashMap of name-value pairs 
     * if deep=true, extracts embedded params, otherwise just top level 
     * params*/
    public static HashMap extractParams(Element xml, boolean deep) {

	if (!xml.getNodeName().equals(PARAM_ELEM+LIST_MODIFIER)) {
	    System.err.println("GSXML:paramList element should have been passed to extractParams, instead it was "+xml.getNodeName());
	    return null;
	}

	NodeList params = null;
	if (deep) { // get all the nested ones
	    params = xml.getElementsByTagName(PARAM_ELEM);
	} else { // just get the top  level ones
	    params = xml.getChildNodes();
	}
	HashMap param_map = new HashMap();
	for (int i=0; i<params.getLength(); i++) {
	    Element param = (Element)params.item(i);
	    String name=param.getAttribute(NAME_ATT);
	    String value=getValue(param); //att or content
	    param_map.put(name, value);
	    
	}
	return param_map;
    }

    /** gets the value att or the text content */
    public static String getValue(Element e) {
	String val = e.getAttribute(VALUE_ATT);
	if (val ==null || val.equals("")) {
	    // have to get it out of the text
	    val=getNodeText(e); 

	}
	return val;
    }
    
    /** extracts the text out of a node */
    public static String getNodeText(Element param) {
	param.normalize();
	Node n = param.getFirstChild();
	while (n!=null && n.getNodeType() !=Node.TEXT_NODE) {
	    n=n.getNextSibling();
	}
	if (n==null) { // no text node
	    return "";
	}
	return n.getNodeValue();
    }

	
    /** add text to a document/subsection  element */
    public static boolean addDocText(Document owner, Element doc, String text) {

	Element content = owner.createElement(NODE_CONTENT_ELEM);
	Text t = owner.createTextNode(text);
	content.appendChild(t);
	doc.appendChild(content);
	return true;
    }
  
    /** add an error message */
    public static boolean addError(Document owner, Element doc, String text) {

	Element content = owner.createElement(ERROR_ELEM);
	Text t = owner.createTextNode(text);
	content.appendChild(t);

	return true;
    }

    /** add an error message */
    public static boolean addError(Document owner, Element doc, Throwable error) {
      error.printStackTrace();
      return addError(owner, doc, error.toString());
    }

    /** adds a metadata elem to a list */
    public static boolean addMetadata(Document owner, Element list, 
				      String meta_name, String meta_value) {
	if (meta_value==null || meta_value.equals("")) {
	    return false;
	}
	Element data = owner.createElement(METADATA_ELEM);
	data.setAttribute(NAME_ATT, meta_name);
	Text t = owner.createTextNode(meta_value);
	data.appendChild(t);
	list.appendChild(data);
	return true;

    }

    /** copies the metadata out of teh metadataList of 'from' into
     * the metadataList of 'to' */
    public static boolean mergeMetadataLists(Node to, Node from) {
	Node to_meta = getChildByTagName(to, METADATA_ELEM+LIST_MODIFIER);
	Node from_meta = getChildByTagName(from, METADATA_ELEM+LIST_MODIFIER);
	
	if  (from_meta == null) { // nothing to copy
	    return true;
	}
	Document to_owner = to.getOwnerDocument();
	Node new_from = to_owner.importNode(from_meta, true);

	if (to_meta == null) { // just copy the whole list
	    to.appendChild(new_from);
	    return true;
	}
	
	// copy individual elements
	Node child = new_from.getFirstChild();
	while ( child != null) {
	    to_meta.appendChild(child);
	    child = child.getNextSibling();
	}
	return true;
    }
	
    /** returns the (first) child element with the given name */
    public static Node getChildByTagName(Node n, String name) {

	Node child = n.getFirstChild();
	while (child!=null) {
	    if (child.getNodeName().equals(name)) {
		return child;
	    }
	    child = child.getNextSibling();
	}
	return null; //not found
    }

    /** takes an xpath type expression of the form name/name/... 
	and returns the first node that matches, or null if not found */
    public static Node getNodeByPath(Node n, String path) {
	
	String link = GSPath.getFirstLink(path);
	path = GSPath.removeFirstLink(path);
	while (!link.equals("")) {
	    n = getChildByTagName(n, link);
	    if (n==null) {
		return null;
	    }
	    link = GSPath.getFirstLink(path);
	    path = GSPath.removeFirstLink(path);
	}
	return n;
    }
    public static HashMap getChildrenMap(Node n) {
	
	HashMap map= new HashMap();
	Node child = n.getFirstChild();
	while (child!=null) {
	    String name = child.getNodeName();
	    map.put(name, child);
	    child = child.getNextSibling();
	}
	return map;
    }
       

    /** Duplicates an element, but gives it a new name */
    public static Element duplicateWithNewName(Document owner, Element element,
					       String element_name, boolean with_attributes)
    {
	Element duplicate = owner.createElement(element_name);

	// Copy element attributes
	if (with_attributes) {
	    NamedNodeMap attributes = element.getAttributes();
	    for (int i = 0; i < attributes.getLength(); i++) {
		Node attribute = attributes.item(i);
		duplicate.setAttribute(attribute.getNodeName(), attribute.getNodeValue());
	    }
	}

	// Copy element children
	NodeList children = element.getChildNodes();
	for (int i = 0; i < children.getLength(); i++) {
	    Node child = children.item(i);
	    duplicate.appendChild(owner.importNode(child, true));
	}

	return duplicate;
    }


    /** returns a basic request message */
    public static  Element createBasicRequest(Document owner, 
					      String request_type, String to, 
					      String lang) {
	Element request = owner.createElement(REQUEST_ELEM);
	request.setAttribute(TYPE_ATT, request_type);
	request.setAttribute(LANG_ATT, lang);
	request.setAttribute(TO_ATT, to);
	return request;
    }
    public static Element createTextElement(Document owner, String elem_name,
					    String text) {
	Element e = owner.createElement(elem_name);
	Text t = owner.createTextNode(text);
	e.appendChild(t);
	return e;

    }


    public static Element createParameter(Document owner, String name,
					  String type, String default_value,
					  String []options) {


	Element p = owner.createElement(PARAM_ELEM);
	p.setAttribute(NAME_ATT, name);
	p.setAttribute(TYPE_ATT, type);
	if (default_value != null) {
	    p.setAttribute(DEFAULT_ATT, default_value);
	}
	if (type.startsWith(PARAM_TYPE_ENUM_START) && options!=null) {
	    for (int i=0; i<options.length; i++) {
		Element e = owner.createElement(PARAM_OPTION_ELEM);
		e.setAttribute(NAME_ATT, options[i]);
		p.appendChild(e);
	    }
	}
	return p;
    }

    /** creates a parm elem containing the text strings
     * <param><name>xxx<name><option name='x'>yyy</option><option name='y'>zzz</option></param>
     */
    public static Element createParameterDisplay(Document owner, String name,
						 String name_text, 
						 String []options,
						 String []option_texts) {
					     

	Element param = owner.createElement(PARAM_ELEM);
	param.setAttribute(NAME_ATT, name);
	param.appendChild(createTextElement(owner, DISPLAY_NAME_ELEM, name_text));
	if (options != null) {
	    for (int i=0; i<options.length; i++) {
		Element e = GSXML.createTextElement(owner, PARAM_OPTION_ELEM, option_texts[i]);
		e.setAttribute(NAME_ATT, options[i]);
		param.appendChild(e);
	    }
	}

    return param;
    }

    /** returns the element parent/node_name[@attribute_name='attribute_value']
     */
    public static Element getNamedElement(Element parent, String node_name,
					  String attribute_name, 
					  String attribute_value) {

	NodeList children = parent.getChildNodes();
	for (int i=0; i<children.getLength(); i++) {
	    Node child = children.item(i);
	    if (child.getNodeName().equals(node_name)) {
		if (((Element)child).getAttribute(attribute_name).equals(attribute_value))
		    return (Element)child;
	    }
	}
	// not found
	return null;
    }


    /** Returns the appropriate language element from a display elem,
     display is the containing element, name is the name of the element to 
     look for, lang is the preferred language, lang_default is the fall back 
     lang if neither lang is found, will return the first one it finds*/
    public static String getDisplayText(Element display, String name,
					   String lang, String lang_default) {
	Element item = getNamedElement(display, name, GSXML.LANG_ATT, lang);
	if (item==null) {
	    item = getNamedElement(display, name, GSXML.LANG_ATT, lang_default);
	}
	if (item ==null) {
	    item = (Element)getChildByTagName(display, name); // just get the first one
	}
	if (item==null) {
	    return ""; // should we return an empty string? or null?
	}
	return getNodeText(item);

    }
    // replaces < > " ' & in the original with their entities
    public static String xmlSafe(String original) {

	StringBuffer filtered = new StringBuffer(original.length());
	char c;
	for (int i=0; i<original.length(); i++) {
	    c = original.charAt(i);
	    if (c == '>') {
		filtered.append("&gt;");
	    } else if (c == '<') {
		filtered.append("&lt;");
	    } else if (c == '"') {
		filtered.append("&quot;");
	    } else if (c == '&') {
		filtered.append("&amp;");
	    } else if (c == '\'') {
		filtered.append("&apos;");
	    } else {
		filtered.append(c);
	    }
	}
	return filtered.toString();
    }
}
